#define __SP_PATH_CHEMISTRY_C__

/*
 * Here are handlers for modifying selections, specific to paths
 *
 * Authors:
 *   Lauris Kaplinski <lauris@kaplinski.com>
 *
 * Copyright (C) 1999-2002 Authors
 * Copyright (C) 2001-2002 Ximian, Inc.
 *
 * Released under GNU GPL, read the file 'COPYING' for more information
 */

#include <string.h>
#include <libart_lgpl/art_misc.h>
#include "xml/repr.h"
#include "svg/svg.h"
#include "sp-path.h"
#include "sp-text.h"
#include "style.h"
#include "sodipodi.h"
#include "document.h"
#include "selection.h"
#include "desktop-handles.h"
#include "path-chemistry.h"
#include "desktop.h"
#include "nodepath.h"
#include "gpc.h"

/* Helper functions for sp_selected_path_to_curves */
static void sp_selected_path_to_curves0 (gboolean do_document_done, guint32 text_grouping_policy);
static SPRepr * sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy);
enum {				
  /* Not used yet. This is the placeholder of Lauris's idea. */
	SP_TOCURVE_INTERACTIVE       = 1 << 0,
	SP_TOCURVE_GROUPING_BY_WORD  = 1 << 1,
	SP_TOCURVE_GROUPING_BY_LINE  = 1 << 2,
	SP_TOCURVE_GROUPING_BY_WHOLE = 1 << 3,
};

void
sp_selected_path_combine (void)
{
	SPDesktop * desktop;
	SPSelection * selection;
	GSList * il;
	GSList * l;
	SPRepr * repr;
	SPItem * item;
	SPPath * path;
	SPCurve * c;
	ArtBpath * abp;
	gchar * d, * str, * style;

	sp_selected_path_to_curves0 (FALSE, 0);

	desktop = SP_ACTIVE_DESKTOP;
	if (!SP_IS_DESKTOP(desktop)) return;
	selection = SP_DT_SELECTION (desktop);

	il = (GSList *) sp_selection_item_list (selection);

	if (g_slist_length (il) < 2) return;

	for (l = il; l != NULL; l = l->next) {
		item = (SPItem *) l->data;
		if (!SP_IS_SHAPE (item)) return;
	}

	il = g_slist_copy (il);

	d = "";
	style = g_strdup (sp_repr_attr ((SP_OBJECT (il->data))->repr, "style"));

	for (l = il; l != NULL; l = l->next) {
		NRMatrixF i2root;
		NRMatrixD i2rootd;
		path = (SPPath *) l->data;
		c = sp_shape_get_curve (SP_SHAPE (path));
		sp_item_i2root_affine (SP_ITEM (path), &i2root);
		nr_matrix_d_from_f (&i2rootd, &i2root);
		abp = art_bpath_affine_transform (c->bpath, NR_MATRIX_D_TO_DOUBLE (&i2rootd));
		sp_curve_unref (c);
		str = sp_svg_write_path (abp);
		art_free (abp);
		d = g_strconcat (d, str, NULL);
		g_free (str);
		sp_repr_unparent (SP_OBJECT_REPR (path));
	}

	g_slist_free (il);

	repr = sp_repr_new ("path");
	sp_repr_set_attr (repr, "style", style);
	g_free (style);
	sp_repr_set_attr (repr, "d", d);
	g_free (d);
	item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), repr);
	sp_document_done (SP_DT_DOCUMENT (desktop));
	sp_repr_unref (repr);

	sp_selection_set_item (selection, item);
}

void
sp_selected_path_break_apart (void)
{
	SPSelection * selection;
	SPRepr * repr;
	SPItem * item;
	SPPath * path;
	SPCurve * curve;
	ArtBpath * abp;
	NRMatrixF i2root;
	NRMatrixD d;
	gchar * style, * str;
	GSList * list, * l;
	SPDesktop * desktop;
	
	desktop = SP_ACTIVE_DESKTOP;
	if (!SP_IS_DESKTOP(desktop)) return;

	selection = SP_DT_SELECTION (desktop);

	item = sp_selection_item (selection);

	if (item == NULL) return;
	if (!SP_IS_PATH (item)) return;

	path = SP_PATH (item);

	curve = sp_shape_get_curve (SP_SHAPE (path));
	if (curve == NULL) return;

	sp_item_i2root_affine (SP_ITEM (path), &i2root);
	nr_matrix_d_from_f (&d, &i2root);
	style = g_strdup (sp_repr_attr (SP_OBJECT (item)->repr, "style"));

	abp = art_bpath_affine_transform (curve->bpath, NR_MATRIX_D_TO_DOUBLE (&d));

	sp_curve_unref (curve);
	sp_repr_unparent (SP_OBJECT_REPR (item));

	curve = sp_curve_new_from_bpath (abp);

	list = sp_curve_split (curve);

	sp_curve_unref (curve);

	for (l = list; l != NULL; l = l->next) {
		curve = (SPCurve *) l->data;

		repr = sp_repr_new ("path");
		sp_repr_set_attr (repr, "style", style);
		str = sp_svg_write_path (curve->bpath);
		sp_repr_set_attr (repr, "d", str);
		g_free (str);
		item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), repr);
		sp_repr_unref (repr);
		sp_selection_add_item (selection, item);
	}
	sp_document_done (SP_DT_DOCUMENT (desktop));

	g_slist_free (list);
	g_free (style);
}

/* This function is an entry point from GUI */
void
sp_selected_path_to_curves (void)
{
	sp_selected_path_to_curves0(TRUE, SP_TOCURVE_INTERACTIVE);
}

static void
sp_selected_path_to_curves0 (gboolean do_document_done, guint32 text_grouping_policy)
{
	SPDesktop *dt;
	SPItem *item;
	SPRepr  *repr;
	
	GSList * il;
	GSList * l;
	SPObject *parent;
	
	dt = SP_ACTIVE_DESKTOP;
	if (!dt) return;
	il = (GSList *) sp_selection_item_list (SP_DT_SELECTION (dt));
	if (!il) return;

	l = il;
	while (l) {
		item = (SPItem *)l->data;
		l    = l->next;
		repr = sp_selected_item_to_curved_repr (item, 0);
		if (!repr)
			continue;
		parent = SP_OBJECT_PARENT (item);
		sp_repr_add_child (SP_OBJECT_REPR (parent), 
				   repr, SP_OBJECT_REPR (item));
		sp_repr_unparent (SP_OBJECT_REPR (item));
		sp_selection_add_repr (SP_DT_SELECTION (dt), repr);
		sp_repr_unref(repr);
	}
	if (do_document_done)
		sp_document_done (SP_DT_DOCUMENT (dt));
}

static SPRepr *
sp_selected_item_to_curved_repr(SPItem * item, guint32 text_grouping_policy)
{
	SPCurve *curve;
	SPRepr  *repr;
	guchar  *style_str;
	guchar  *def_str;
	
	if (!item)
		return NULL;
		
	if (SP_IS_SHAPE (item)) {
		curve = sp_shape_get_curve (SP_SHAPE (item));
	} else if (SP_IS_TEXT (item)) {
		curve = sp_text_normalized_bpath (SP_TEXT (item));
	} else {
		curve = NULL;
	}
	
	if (!curve) return NULL;
	
	repr = sp_repr_new ("path");
	/* Transformation */
	sp_repr_set_attr (repr, "transform", 
			  sp_repr_attr (SP_OBJECT_REPR (item), "transform"));
	/* Style */
	style_str = sp_style_write_difference (SP_OBJECT_STYLE (item), 
					       SP_OBJECT_STYLE (SP_OBJECT_PARENT (item)));
	sp_repr_set_attr (repr, "style", style_str);
	g_free (style_str);
	/* Definition */
	def_str = sp_svg_write_path (curve->bpath);
	sp_repr_set_attr (repr, "d", def_str);
	g_free (def_str);
	sp_curve_unref (curve);
	return repr;
}

void
sp_path_cleanup (SPPath *path)
{
	SPCurve *curve;
	GSList *curves, *c;
	SPStyle *style;
	gboolean dropped;

	if (strcmp (sp_repr_name (SP_OBJECT_REPR (path)), "path")) return;

	style = SP_OBJECT_STYLE (path);
	if (style->fill.type == SP_PAINT_TYPE_NONE) return;

	curve = sp_shape_get_curve (SP_SHAPE (path));
	if (!curve) return;
	c = sp_curve_split (curve);
	sp_curve_unref (curve);

	dropped = FALSE;
	curves = NULL;
	while (c) {
		curve = (SPCurve *) c->data;
		if (curve->closed) {
			curves = g_slist_prepend (curves, curve);
		} else {
			dropped = TRUE;
			sp_curve_unref (curve);
		}
		c = g_slist_remove (c, c->data);
	}
	curves = g_slist_reverse (curves);

	curve = sp_curve_concat (curves);

	while (curves) {
		sp_curve_unref ((SPCurve *) curves->data);
		curves = g_slist_remove (curves, curves->data);
	}

	if (sp_curve_is_empty (curve)) {
		sp_repr_unparent (SP_OBJECT_REPR (path));
	} else if (dropped) {
		guchar *svgpath;
		svgpath = sp_svg_write_path (curve->bpath);
		sp_repr_set_attr (SP_OBJECT_REPR (path), "d", svgpath);
		g_free (svgpath);
	}

	sp_curve_unref (curve);
}

static void
sp_selected_path_normalize_intersect (SPNodePath * np1, SPNodePath * np2)
{
	GSList * spl, * nl, * pn1, * pn2;
	GSList * pnl1, * pnl2;
	SPPathNode * n;
	SPPathNode * i, * j;
	SPNodeSubPath * sp;

	if (!np1 || !np2)
		return;

	pnl1 = NULL;
	pnl2 = NULL;

	for (spl = np1->subpaths; spl != NULL; spl = spl->next) {
		sp = (SPNodeSubPath *) spl->data;
		
		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			n = (SPPathNode *) nl->data;
			pnl1 = g_slist_prepend (pnl1, n);
		}

	}

	for (spl = np2->subpaths; spl != NULL; spl = spl->next) {
		sp = (SPNodeSubPath *) spl->data;
		
		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			n = (SPPathNode *) nl->data;
			pnl2 = g_slist_prepend (pnl2, n);
		}

	}

	for (pn1 = pnl1; pn1 != NULL; pn1 = pn1->next) {
		i =  (SPPathNode *) pn1->data;

		for (pn2 = pnl2; pn2 != NULL; pn2 = pn2->next) {
			j =  (SPPathNode *) pn2->data;

			if (!i || !j)
				goto end;

			if (i->code != ART_CURVETO && j->code != ART_CURVETO)
				continue;

			if (sp_nodepath_add_node_intersect(i, j, 3)) {
				sp_selected_path_normalize_intersect(np1, np2);
				goto end;
			}
		}
	}

 end:
	g_slist_free (pnl1);
	g_slist_free (pnl2);
}

void 
sp_selected_path_normalize_intersect_check (SPNodePath * np1, SPNodePath * np2)
{
	GSList * spl, * nl, * pn1, * pn2;
	GSList * pnl1, * pnl2;
	SPPathNode * n;
	SPPathNode * i, * j;
	SPNodeSubPath * sp;
	NRPointF intersect;

	if (!np1 || !np2)
		return;

	pnl1 = NULL;
	pnl2 = NULL;

	for (spl = np1->subpaths; spl != NULL; spl = spl->next) {
		sp = (SPNodeSubPath *) spl->data;
		
		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			n = (SPPathNode *) nl->data;
			pnl1 = g_slist_prepend (pnl1, n);
		}

	}

	for (spl = np2->subpaths; spl != NULL; spl = spl->next) {
		sp = (SPNodeSubPath *) spl->data;
		
		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			n = (SPPathNode *) nl->data;
			pnl2 = g_slist_prepend (pnl2, n);
		}

	}

	for (pn1 = pnl1; pn1 != NULL; pn1 = pn1->next) {
		i =  (SPPathNode *) pn1->data;

		for (pn2 = pnl2; pn2 != NULL; pn2 = pn2->next) {
			j =  (SPPathNode *) pn2->data;

			if (i->code != ART_CURVETO && j->code != ART_CURVETO)
				continue;

			if (sp_intersector_segment_intersect(i->pos, i->n.other->pos, j->pos, j->n.other->pos, &intersect) != 0
			    || hypot (intersect.x - i->pos.x, intersect.y - i->pos.y) < 0.1 
			    || hypot (intersect.x - i->n.other->pos.x, intersect.y - i->n.other->pos.x) < 0.1
			    || hypot (intersect.x - j->pos.x, intersect.y - j->pos.y) < 0.1
			    || hypot (intersect.x - j->n.other->pos.x, intersect.y - j->n.other->pos.y) < 0.1)
				continue;

			if (i->code == ART_CURVETO) {
				sp_nodepath_line_add_node (i->n.other, 0.5);
			} else {
				sp_nodepath_line_add_node (j->n.other, 0.5);
			}
			update_repr_no_undo (np1);
			update_repr_no_undo (np2);
			sp_selected_path_normalize_intersect_check (np1, np2);
			goto end;
		}
	}

 end:
	g_slist_free (pnl1);
	g_slist_free (pnl2);

}

static void
sp_path_to_gpc_polygon (SPNodePath * np, gpc_polygon * polygon)
{
	int i, j;
	GSList * spl, * nl;
	SPPathNode * n;
	SPNodeSubPath * sp;

	if (!np || !polygon)
		return;

	polygon->num_contours = g_slist_length (np->subpaths);
	polygon->hole = g_new0 (int, polygon->num_contours);
	polygon->contour = g_new (gpc_vertex_list, polygon->num_contours);

	i = 0;
	for (spl = np->subpaths; spl != NULL; spl = spl->next) {
		gpc_vertex_list * gvl = &polygon->contour[i];
		
		sp = (SPNodeSubPath *) spl->data;

		gvl->num_vertices = g_slist_length(sp->nodes);
		gvl->vertex = g_new (gpc_vertex, gvl->num_vertices);
		
		j = 0;
		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			n = (SPPathNode *) nl->data;
			gvl->vertex[j].x = n->pos.x ;
			gvl->vertex[j].y = n->pos.y ;
			j++;
		}
	}
}

static gchar *
sp_gpc_polygon_to_repr (SPItem * parent, gpc_polygon * polygon)
{
	ArtBpath * bpath;
	gchar * str;
	NRMatrixF tmp;
	NRMatrixD doc2dt;
	NRMatrixD d2i, i2d;

	int size;
	int c, v, i;
	gfloat prev_x, prev_y;

	size = 1;
	for (c = 0; c < polygon->num_contours; c++) {
		size += polygon->contour[c].num_vertices;
 	}

	bpath = g_new0 (ArtBpath, size);

	nr_matrix_f_set_identity (&tmp);
	nr_matrix_d_set_scale (&doc2dt, 0.8, -0.8);
	doc2dt.c[5] = sp_document_height (SP_OBJECT_DOCUMENT (parent));
	nr_matrix_multiply_ffd (&tmp, &tmp, &doc2dt);
	nr_matrix_d_from_f (&i2d, &tmp);
	nr_matrix_d_invert (&d2i, &i2d);

	i = 0;
	for (c = 0; c < polygon->num_contours; c++) {
		for (v = 0; v < polygon->contour[c].num_vertices; v++) {

			if (i>0) {
				if ( hypot(prev_x - polygon->contour[c].vertex[v].x, 
					   prev_y - polygon->contour[c].vertex[v].y) < 0.001)
					continue;
			}

			if (v==0)
				bpath[i].code = ART_MOVETO;
			else 
				bpath[i].code = ART_LINETO;

			prev_x = polygon->contour[c].vertex[v].x;
			prev_y = polygon->contour[c].vertex[v].y;
			bpath[i].x3 = NR_MATRIX_DF_TRANSFORM_X (&d2i, prev_x, prev_y);
			bpath[i].y3 = NR_MATRIX_DF_TRANSFORM_Y (&d2i, prev_x, prev_y);
			i++;
		}
	}

	bpath[i].code = ART_END;

	str = sp_svg_write_path (bpath);

	g_free (bpath);

	return str;
}

SPPathNode *
get_same_pathnode_inverse_in (SPPathNode * searched, SPNodePath * np1, SPNodePath * np2)
{
	GSList * spl, * nl;
	SPPathNode * pn;
	SPNodeSubPath * sp;

	if (!np1 || !np2 || !searched)
		return;

	for (spl = np1->subpaths; spl != NULL; spl = spl->next) {

		sp = (SPNodeSubPath *) spl->data;

		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			pn = (SPPathNode *) nl->data;

			if ( hypot (searched->pos.x - pn->pos.x, searched->pos.y - pn->pos.y) < 0.001
			     && hypot (searched->n.other->pos.x - pn->p.other->pos.x, searched->n.other->pos.y - pn->p.other->pos.y) < 0.001 )
				return pn;
			/*
			fprintf (stderr, "inv 1: (%f,%f)!=(%f,%f) && (%f,%f)!=(%f,%f)\n",
				 searched->pos.x, searched->pos.y, pn->pos.x, pn->pos.y, 
				 searched->n.other->pos.x, searched->n.other->pos.y, pn->p.other->pos.x, pn->p.other->pos.y);
			*/
		}
	}

	for (spl = np2->subpaths; spl != NULL; spl = spl->next) {

		sp = (SPNodeSubPath *) spl->data;

		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			pn = (SPPathNode *) nl->data;

			if ( hypot (searched->pos.x - pn->pos.x, searched->pos.y - pn->pos.y) < 0.001
			     && hypot (searched->n.other->pos.x - pn->p.other->pos.x, searched->n.other->pos.y - pn->p.other->pos.y) < 0.001 )
				return pn;

			/*
			fprintf (stderr, "inv 2: (%f,%f)!=(%f,%f) && (%f,%f)!=(%f,%f)\n",
				 searched->pos.x, searched->pos.y, pn->pos.x, pn->pos.y, 
				 searched->n.other->pos.x, searched->n.other->pos.y, pn->p.other->pos.x, pn->p.other->pos.y);
			*/
		}
	}

	return NULL;
}

SPPathNode *
get_same_pathnode_in (SPPathNode * searched, SPNodePath * np1, SPNodePath * np2)
{
	GSList * spl, * nl;
	SPPathNode * pn;
	SPNodeSubPath * sp;

	if (!np1 || !np2 || !searched)
		return;

	for (spl = np1->subpaths; spl != NULL; spl = spl->next) {

		sp = (SPNodeSubPath *) spl->data;

		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			pn = (SPPathNode *) nl->data;

			if ( hypot (searched->pos.x - pn->pos.x, searched->pos.y - pn->pos.y) < 0.001
			     && hypot (searched->n.other->pos.x - pn->n.other->pos.x, searched->n.other->pos.y - pn->n.other->pos.y) < 0.001 )
				return pn;
			/*
			fprintf (stderr, "1: (%f,%f)!=(%f,%f) && (%f,%f)!=(%f,%f)\n",
				 searched->pos.x, searched->pos.y, pn->pos.x, pn->pos.y, 
				 searched->n.other->pos.x, searched->n.other->pos.y, pn->n.other->pos.x, pn->n.other->pos.y);
			*/
		}
	}

	for (spl = np2->subpaths; spl != NULL; spl = spl->next) {

		sp = (SPNodeSubPath *) spl->data;

		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			pn = (SPPathNode *) nl->data;

			if ( hypot (searched->pos.x - pn->pos.x, searched->pos.y - pn->pos.y) < 0.001
			     && hypot (searched->n.other->pos.x - pn->n.other->pos.x, searched->n.other->pos.y - pn->n.other->pos.y) < 0.001 )
				return pn;

			/*
			fprintf (stderr, "2: (%f,%f)!=(%f,%f) && (%f,%f)!=(%f,%f)\n",
				 searched->pos.x, searched->pos.y, pn->pos.x, pn->pos.y, 
				 searched->n.other->pos.x, searched->n.other->pos.y, pn->n.other->pos.x, pn->n.other->pos.y);
			*/
		}
	}

	return NULL;
}

static void
sp_nodepath_rearrange (SPNodePath * np1, SPNodePath * np2, SPNodePath * result)
{
	GSList * spl, * nl;
	SPPathNode * pn, * same;
	SPNodeSubPath * sp;
	SPCurve * curve;
	gchar * svgpath;

	if (!np1 || !np2 || !result)
		return;

	for (spl = result->subpaths; spl != NULL; spl = spl->next) {

		sp = (SPNodeSubPath *) spl->data;

		for (nl = sp->nodes; nl != NULL; nl = nl->next) {
			pn = (SPPathNode *) nl->data;
			if ((same = get_same_pathnode_in (pn, np1, np2))) {
				/*
				fprintf (stderr, "same found (%f,%f)->(%f,%f)\n",
					 same->pos.x, same->pos.y, same->n.other->pos.x, same->n.other->pos.y);
				*/

				pn->code = ART_CURVETO;
				pn->n.other->code = ART_CURVETO;
				pn->type = SP_PATHNODE_CUSP;
				pn->n.other->type = SP_PATHNODE_CUSP;
				pn->n.pos = same->n.pos;
				pn->n.other->p.pos = same->n.other->p.pos;

			} else if ((same = get_same_pathnode_inverse_in (pn, np1, np2))) {
				/*
				fprintf (stderr, "inv same found (%f,%f)->(%f,%f)\n",
					 same->pos.x, same->pos.y, same->p.other->pos.x, same->p.other->pos.y);
				*/

				pn->code = ART_CURVETO;
				pn->n.other->code = ART_CURVETO;
				pn->type = SP_PATHNODE_CUSP;
				pn->n.other->type = SP_PATHNODE_CUSP;
				pn->n.pos = same->p.pos;
				pn->n.other->p.pos = same->p.other->n.pos;
			} else {
				fprintf (stderr, "NOT FOUND (%f,%f)\n", pn->pos.x, pn->pos.y);
			}
		}
	}

	update_repr_no_undo (result);
}

static void
sp_selected_path_bool_op (gpc_op op)
{
	SPDesktop * desktop;
	SPSelection * selection;
	GSList * il, * selected;
	gchar * style;
	gchar * str;
	SPRepr * repr;
	SPItem * item;
	SPPath * path;
	SPNodePath * np1, * np2, * result;
	SPNodeSubPath * sp;
	gpc_polygon subject_polygon, clip_polygon, result_polygon;

	sp_selected_path_to_curves0 (FALSE, 0);

	desktop = SP_ACTIVE_DESKTOP;
	if (!SP_IS_DESKTOP(desktop)) return;
	selection = SP_DT_SELECTION (desktop);

	il = (GSList *) sp_selection_item_list (selection);

	if (g_slist_length (il) != 2) return;

	item = (SPItem *) il->data;
	if (!SP_IS_SHAPE (item)) return;
	if (!(np1 = sp_nodepath_new (desktop, item)))
		return;

	item = (SPItem *) il->next->data;
	if (!SP_IS_SHAPE (item)) return;
	if (!(np2 = sp_nodepath_new (desktop, item))) {
		sp_nodepath_destroy (np1);
		return;
	}

	sp_selected_path_normalize_intersect (np1, np2);
	sp_selected_path_normalize_intersect_check (np1, np2);
	sp_nodepath_destroy (np1);
	sp_nodepath_destroy (np2);

	selection = SP_DT_SELECTION (desktop);

	il = (GSList *) sp_selection_item_list (selection);
	item = (SPItem *) il->data;
	np1 = sp_nodepath_new (desktop, item);
	item = (SPItem *) il->next->data;
	np2 = sp_nodepath_new (desktop, item);

	sp_path_to_gpc_polygon (np1, &subject_polygon);
	sp_path_to_gpc_polygon (np2, &clip_polygon);

	gpc_polygon_clip (op, &subject_polygon, &clip_polygon, &result_polygon);

	if (result_polygon.num_contours > 0) {
		item = (SPItem *) il->data;
		style = g_strdup (sp_repr_attr (SP_OBJECT (item)->repr, "style"));
		repr = sp_repr_new ("path");
		sp_repr_set_attr (repr, "style", style);

		str = sp_gpc_polygon_to_repr (item, &result_polygon);

		sp_repr_set_attr (repr, "d", str);
		item = (SPItem *) sp_document_add_repr (SP_DT_DOCUMENT (desktop), repr);
		sp_repr_unref (repr);

		g_free (str);
		g_free (style);

		result = sp_nodepath_new (desktop, item);
		sp_nodepath_rearrange (np1, np2, result);
	}

	/* Deleting origin polygon */
	selected = g_slist_copy ((GSList *) sp_selection_repr_list (selection));
	sp_selection_empty (selection);

	while (selected) {
		sp_repr_unparent ((SPRepr *) selected->data);
		selected = g_slist_remove (selected, selected->data);
	}

	if (result_polygon.num_contours > 0)
		sp_selection_add_item (selection, item);

	sp_document_done (SP_DT_DOCUMENT (desktop));

	sp_nodepath_destroy (np1);
	sp_nodepath_destroy (np2);
	sp_nodepath_destroy (result);

	gpc_free_polygon (&subject_polygon);
	gpc_free_polygon (&clip_polygon);
	gpc_free_polygon (&result_polygon);
}

void
sp_selected_path_bool_intersect (void)
{
	sp_selected_path_bool_op (GPC_INT);
}

void
sp_selected_path_bool_union (void)
{
	sp_selected_path_bool_op (GPC_UNION);
}

void
sp_selected_path_bool_difference (void)
{
	sp_selected_path_bool_op (GPC_DIFF);
}

void
sp_selected_path_bool_xor (void)
{
	sp_selected_path_bool_op (GPC_XOR);
}
